/*
   ***********************************************
   *             MAZE GAME 1/(infinity)          *
   *                  BASIC TRAINING             *
   *                    ver 1.1                  *
   *               by Shawn Henderson            *
   *                                             *
   ***********************************************

I suggest that you, the user, read this entire comment
page, as it will explain sources to some code, give
(or take) acknowlegements and clarify some of the
techniques used in this program.


SPRITES:
-------

First of all, all tiles and sprites are created with a
utility called SEDIT, or Sprite Edit, which was also
created by me.  It is a utility focussing on flexibility.
I knew that I wouldn't be the only one using this program.
As soon as others saw the results, they wanted to try it
for themselves.

To this end, I have included a semi-complte help screen to
aid with the editors use, as well as quite a few drawing
functions like line, polygon fill, color replace and
auto-mask.

The file format for the tile is simple, the first 2 bytes
are the integer for the X size of the sprite, next 1 byte for
a space and then another integer for the Y size.  (NOTE: I
could have used the DOS write and read routines to get byte
by byte accuracy, save space and be generally more efficent,
but I reasoned that others may want to edit the TSV (Tile Save)
files directly with EDIT.COM, so I left them in plain ASCII
format)
The next is a grid of numbers, X * Y, from 0 to 16 representing
the color numbers of the pixels that go in that part of the
grid.  0 to 15 are the color numbers of the default BIOS
palette, 0 being black, 15 being white and so on,
the 16 is a numeric representation of a color that is
transparent, so that when the sprite is PUT down on the screen,
I can allow bits to not be blacked out like they would with a
simple GETIMAGE and PUTIMAGE (more on that later)
As for SEDIT, any problems with it I respond to and I
help others with it's use.  CPP file is availiable to
others on demand.

MAPS:
----

I, in an attempt to see how the tiles fit together, created a
little program that eventually evolved into a full fledged
map maker.  One of my eariler versions was released to others
for thier projects, but as I realised how specific a map needs
to be to it's respective interpreter, my version is signifigantly
customized to my needs.  I only give away the CPP to this program
on request, and rarely offer help editing it.

My map file format is rather interesting, first off, it's
like the SEDIT TSV format, in the way that it is, again,
simple ASCII text and not in binary, for conveinece to
myself and others.  The first 2 numbers are the X and Y
lengths + 1, as 0 is not counted.

Again, like TSV, there is a grid, each number represents the
TILE in the tile set, this is to be distinguised from a SPRITE
(more on that later) after that, a single number followed by
a CR and LF, being the number of enemies on the map.

Then, this pattern is repeated for whatever number of times
number of enemy was:
enemyx position, enemyy position, enemy SPRITE number.

Then, another 12 numbers each seperated by CR LF, each being
a percentage probablility (thank you Finite) of any of the 12
powerups appearing in the map.

Then a final number, being the number of seconds it took me,
under a random cercumstance, to finish the level, which is
used as a benchmark time.



TILE vs. SPRITE:
---------------

There is a small difference between a TILE and a SPRITE, it
that the SPRITE has transparent data (ie, 16) and a TILE
doesn't.  It is, for this reason, why there are 4 routines
for the getting and putting of images.

VOID getimg takes 2 parameters, a number as a holder, and a
string as a filename.  The routine quickly opens up the TSV
file, and draws it pixel by pixel to the screen (This is not
seen by way of setting the entire palette to black, and then
resetting it once everything is loaded) and then simply
GETIMAGE'ing it into a far pointer, where n is the element
of that arrayed pointer.

VOID putimg takes 3 parameters, the holder element of the
arrayed pointer to PUTIMAGE from, the x coord on the screen
and the Y coord.


Now, VOID getsprite and VOID putsprite were a little
different earlier, thier method of holding was through
a char array.   Since the 16 (trans) data needed to be
retained, I couldn't GET and PUTIMAGE it, so in the
beginning I had a char array, that was 3 dimensional.
The first dimension was the number of tiles, like the
arrayed pointer holder.  The two other dimensions
were always [33][33], to have the room to hold
a 32*32 sprite as numbers, and since the numbers
never exceeded 16, a single byte would do, so the
TILES array became an char array.

Now, this worked, up until I started having many
tiles, and a problem arose, when I got up to around
50, that works out to this as a statement:
TILES[50][33][33] (don't ask why it's 33*33 as opposed
to 32*32, it just doesn't work that way), but as a
char array, that still works out to 54,450 bytes, and
the DGROUP, which can't exceed 64k (65536) (and which
is the holder for global variables, of which TILES was)
I ran out of room, with all the other global variables,
structures and all, I was really pushing that limit.

But after playing around with far pointers, the
ALLOC.H fuctions and the MOVEDATA routine, I have
now come up with a work around.

I only need one conventional memory array to hold
the active sprite, a simple char [33][33] one will
do.  I have set up another arrayed far pointer that
holds the data.  I allocate, when called for,
1100 bytes for the data (33*33 + 11 for good measure)
then I just move the data directly from that array to
far pointer, and I do that for all the sprites on game
loadup.  Then, when a sprite is needed to be drawn,
in the PUTSPRITE routine, it quickly copies back the
data from the far pointer (the numbers from the old
array and the new pointer match for the moment)
to the temp holder in conventional memory that
was originally used to hold the data when FSCANF'ing
in, and from there, the routine continues normally and
the sprite is put on the screen.  I haven't noticed
a decrease in speed due to this, and the blitting
works well with other things too.  I was just lucky
that the computer thinks like I do.  The elements
of an array are sequencially stored.  Example, if you
had an array of 5 integers, (0 to 4 elements) and you
were trying to move the 2nd integer, you already know
that the first ones segment and offset will be element
0's, so you just add the number of elements and the
bytes per element to find the new offset.



SPECIAL EFFECTS:
---------------

(The following effects were created and executed by myself,
 and the only credit to be given is to the HELP files with
 respect to syntax's of the commands and some example code)

The title screen was originally created in QBASIC, as a
bunch of screen mode 12 (640*480*16) primitives, and saved
to a file, but when I discovered that this file was too large
and loaded too slow to be practical, I just did a conversion
to the C primitives.

The text in the instructions was done using palette
manipulation and a few other basic techniques.
The melting effect is an old trick converted from BASIC to C.

The 3D text in the menu was simple and has no point.

I have my own timing counter.  It was created due to the
problem of pausing the game.  Since, up until recently,
all the enemies delays and powerup delays were handled
by the time_t structure and using time(NULL), any
pausing of the game would not pause "time", as it were.
So I simply created a variable to act as time(NULL), that
increased every second, but if something were to interupt
that increase, the game, was in effect, paused.



MUSIC:
-----

The music routines were quite difficult to make
and still, to this date, fail to please me.  They
were converted from QBASIC's PLAY statement with
painstakeing effort and musical ear.

Notes C through B in octave 1 work the best, but from
there (0, 2, 3) the notes begin to sound bad, this is
due to my lack in disovering the formula that produces
the proper frequences for use with the sound statement.

The pauses, note lengths other delays were converted through
converting QBASIC's PLAY ticker (computer clock, 18.2
ticks/s) to the milliseconds that C uses.



*/





#include <graphics.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <alloc.h>
#include <string.h>
#include <time.h>
#include <dos.h>
#include <mem.h>







struct {
int notealong, musiclen, notelen, noteoct, delays, freq, pause, anymusic;
} musictype;

struct my_struct {
char enemyx, enemyy, rand, alive, enemydir;
char stun, waittime, stuntime, ifstunned;
char enemy, bonus;
int waitforit;
} emetype[20];


float numtilew = 0;

int exitmapx;
int exitmapy;
int exitposx;
int exitposy;

int mapposx = 3;
int mapposy = 3;
int goodtime;
char layoutmap[6][6];
char levellookup[6][6];
char beenthere[6][6];
//char tiles[47][32][32];

char music[200];
void far *holdtiles[100];
char temptile[33][33];
void far *ptr[50];
int bools;
char map[18][16];
int movex, movey;
int tilex, tiley;
char choice;
int bools2;
int maxx, maxy;
int numtiles, memit;
int playerpic;
int score = 0;
int numenemy;
int tileflipped = 2;
int dotalive;
int offsetx, offsety;
int temp;
int pstunned;
int holdtime = 2;
int numkey = 3;
int numlives = 5;
int beatlevel = 0;
int probs[15];
int havepowerup;
int oldpowerup;
int powerup = 0;
int px, py;
int powerdelay = 2;
int powerstay = 5;
int poweralive;
int powerappear;
int powerhold;
int pstun;
time_t gametime;
int timetoplay = 0;
int havecomp;

void quitgame();
void main()
{
//struct my_struct emetype[10];
musictype.anymusic = 1;
void getimg(int n, char name[15]);
void putimg(int n, int x, int y, int type);
void dscreen();
void loadmap(char fname[15]);
void putsprite(int n, int x, int y);
void getsprite(int n, char name[15]);
void fliptile(int thex, int they, int thet);
void handleenemy();
void usepower(int thep);
void getputquick(int x, int y, char name[30]);
void initgame();
void menu();
void win();
int getprob();
void start();
void instruct();
void musicplay();
void musicnotestep();
void initmusic(char temp[200]);



int i, j;
tilex = 32; tiley = 32;
maxx = 16;
maxy = 12;
offsetx = 16;
offsety = 32;
dotalive = 1;
pstunned = 0;




for (j = 1; j <= maxy; j++)
 for (i = 1; i <= maxx; i++)
  map[i][j] = 0;



int gdriver = DETECT, gmode;
initgraph(&gdriver, &gmode, "");


initgame();
movex = 8;
movey = 1;

start();

newlevel:
timetoplay = 0;



randomize();

char mapname[20];



itoa(levellookup[mapposx][mapposy], mapname, 10);
strcat(mapname, ".map");
//printf ("*** %s ***", mapname);
//getch();
//loadmap(mapname);
loadmap(mapname);
cleardevice();
dscreen();
playerpic = 1;

for (i = 1; i <= numenemy; i++)
  emetype[i].waitforit = timetoplay;
pstun = timetoplay;
powerappear = timetoplay;
gametime = timetoplay;

getputquick (580, 80, "lifesign.tsv");
getputquick (580, 115, "pow12.tsv");

rectangle (578, 38, 614, 72);
for ( ; ; )
{

if (time(NULL) - gametime >= 1 && beatlevel == 0)
  {
    gametime = time(NULL);
    timetoplay++;
  }


gotoxy (78,7);
printf ("%d", numlives);
gotoxy (78,9);
printf ("%d", numkey);
gotoxy (73,11);
printf ("S: %d", score);


setcolor(15);
putsprite (playerpic, movex*tilex+offsetx, movey*tiley+offsety);



if (score >= 0 && score < 1000)
 holdtime = 2;

if (score >= 1000 && score < 3000)
 holdtime = 3;

if (score >= 3000 && score < 5000)
 holdtime = 4;

if (score >= 5000 && score < 10000)
 holdtime = 5;

if (score >= 10000)
 holdtime = 10000;


if (timetoplay - powerappear >= powerdelay && powerup == 0 && beatlevel == 0)
 {
   powerup = getprob() + 35+4;
   px = 0; py = 0;
   while (map[px][py] > 2)
     {
       px = random(maxx);
       py = random(maxy);
    }
   poweralive = timetoplay;

 }

 if (timetoplay - poweralive >= powerstay && powerup != 0)
   {
    powerup = 0;
    putimg (map[px][py], px*tilex+offsetx, py*tiley+offsety, 0);
    powerappear = timetoplay;
    }

  if (powerup != 0)
    putsprite (powerup, px*tilex+offsetx, py*tiley+offsety);

  if (px == movex && py == movey && powerup != 0)
    {
    while (powerup == 42+4)
      if (powerup == 42+4) powerup = getprob() + 35+4;

       oldpowerup = powerup;
     if (powerup == 38+4)
       numlives++;
     else if (powerup == 39+4)
        havecomp = 1;
     else if (powerup == 46+4)
        numkey++;
     else if (powerup == 44+4)
        score = score + random(50)+50;
     else if (powerup == 45+4)
        score = score + random(300)+ 200;
     else
      havepowerup = powerup;

     powerup = 0;
     powerappear = timetoplay;
     putimg (map[movex][movey], movex*tilex+offsetx, movey*tiley+offsety, 0);
     powerhold = timetoplay;
     }

     if (timetoplay - powerhold >= holdtime)
       {
         havepowerup = 0;
         putsprite (0, 580, 40);
       }

if (havepowerup != 0)
  putsprite (havepowerup, 580, 40);




choice = 0;
if (kbhit())
  choice = getch();

if (choice != 0)
 {
//  map[movex][movey] = 2;
	putimg (map[movex][movey], movex*tilex+offsetx, movey*tiley+offsety, 0);
   }


if (timetoplay - pstun >= pstunned && pstunned != 0)
  {
    putimg (map[movex][movey], movex*tilex+offsetx, movey*tiley+offsety, 0);
    pstunned = 0;
    playerpic = 1;
  }


if (dotalive != 0 && pstunned == 0)
{

  if (choice == 80)
  {
	playerpic = 1;


   if (map[movex][movey+1] >= 4 && map[movex][movey+1] <= 7 && numkey > 0)
     {
      if (movex == 8 && movey == 1)
             {
             map[8][0] = 8;
             putimg(8, 8*tilex+offsetx, 0+offsety,0);
             }

      if (movex == 8 && movey == 11)
             {
             map[8][12] = 9;
             putimg(9, 8*tilex+offsetx, 12*tiley+offsety,0);
             }

     if (movex == 1 && movey == 6)
            {
             map[0][6] = 10;
             putimg(10, 0*tilex+offsetx, 6*tiley+offsety,0);
             }

     if (movex == 15 && movey == 6)
             {
             map[16][6] = 11;
             putimg(11, 16*tilex+offsetx, 6*tiley+offsety,0);
             }

     movey--;
      numkey--;
     }


   if (map[movex][movey+1] >= 8 && map[movex][movey+1] <= 11)
     {


     if (exitposx == mapposx && exitposy == mapposy && exitmapx == movex && exitmapy == movey)
        win();
       cleardevice();
       mapposy++;
       if (mapposy > 5) mapposy = 1;
       movey = 1;
       goto newlevel;
      }

	if (map[movex][movey+1] < 3)
		movey++;
	if (movey > maxy) movey = maxy;
  }
if (choice == 72)
  {
	  playerpic = 4;

      if (map[movex][movey-1] >= 4 && map[movex][movey-1] <= 7 && numkey > 0)
     {
      if (movex == 8 && movey == 1)
             {
             map[8][0] = 8;
             putimg(8, 8*tilex+offsetx, 0+offsety,0);
             }

      if (movex == 8 && movey == 11)
             {
             map[8][12] = 9;
             putimg(9, 8*tilex+offsetx, 12*tiley+offsety,0);
             }

     if (movex == 1 && movey == 6)
            {
             map[0][6] = 10;
             putimg(10, 0*tilex+offsetx, 6*tiley+offsety,0);
             }

     if (movex == 15 && movey == 6)
             {
             map[16][6] = 11;
             putimg(11, 16*tilex+offsetx, 6*tiley+offsety,0);
             }

      movey++;
      numkey--;
     }







      if (map[movex][movey-1] >= 8 && map[movex][movey-1] <= 11)
     {

     if (exitposx == mapposx && exitposy == mapposy && exitmapx == movex && exitmapy == movey)
       win();


       cleardevice();
       mapposy--;
       if (mapposy < 1) mapposy = 5;
       movey = 11;
      goto newlevel;
      }

   if (map[movex][movey-1] < 3)
	movey--;
	if (movey < 0) movey = 0;
  }
if (choice == 77)
  {
	  playerpic = 2;

      if (map[movex+1][movey] >= 4 && map[movex+1][movey] <= 7 && numkey > 0)
     {
      if (movex == 8 && movey == 1)
             {
             map[8][0] = 8;
             putimg(8, 8*tilex+offsetx, 0+offsety,0);
             }

      if (movex == 8 && movey == 11)
             {
             map[8][12] = 9;
             putimg(9, 8*tilex+offsetx, 12*tiley+offsety,0);
             }

     if (movex == 1 && movey == 6)
            {
             map[0][6] = 10;
             putimg(10, 0*tilex+offsetx, 6*tiley+offsety,0);
             }

     if (movex == 15 && movey == 6)
             {
             map[16][6] = 11;
             putimg(11, 16*tilex+offsetx, 6*tiley+offsety,0);
             }

      movex--;
      numkey--;
     }




     if (map[movex+1][movey] >= 8 && map[movex+1][movey] <= 11)
     {
     if (exitposx == mapposx && exitposy == mapposy && exitmapx == movex && exitmapy == movey)
       win();

       cleardevice();
      mapposx++;
     if (mapposx > 5) mapposx = 1;
      movex = 1;
      goto newlevel;
      }

  if (map[movex+1][movey] < 3)
	movex++;
	if (movex > maxx) movex = maxx;
  }
if (choice == 75)
  {
	  playerpic = 3;


    if (map[movex-1][movey] >= 4 && map[movex-1][movey] <= 7 && numkey > 0)
     {
      if (movex == 8 && movey == 1)
             {
             map[8][0] = 8;
             putimg(8, 8*tilex+offsetx, 0+offsety,0);
             }

      if (movex == 8 && movey == 11)
             {
             map[8][12] = 9;
             putimg(9, 8*tilex+offsetx, 12*tiley+offsety,0);
             }

     if (movex == 1 && movey == 6)
            {
             map[0][6] = 10;
             putimg(10, 0*tilex+offsetx, 6*tiley+offsety,0);
             }

     if (movex == 15 && movey == 6)
             {
             map[16][6] = 11;
             putimg(11, 16*tilex+offsetx, 6*tiley+offsety,0);
             }

      movex++;
      numkey--;
     }







    if (map[movex-1][movey] >= 8 && map[movex-1][movey] <= 11)
     {

     if ((exitposx == mapposx) && (exitposy == mapposy) && (exitmapx == movex) && (exitmapy == movey))
       win();

       cleardevice();
       mapposx--;
       if (mapposx < 1) mapposx = 5;
       movex = 15;
       goto newlevel;
         }


  if (map[movex-1][movey] < 3)
	movex--;
	if (movex < 0) movex = 0;
  }

}

if (choice == 'l')
  {
  gotoxy (1,1);
  for (j = 0; j <= maxy; j++)
    {
   for (i = 0; i <= maxx; i++)
    printf ("%d ", map[i][j]);

    printf ("\n");
     }

  printf ("\n\n\n");

   for (j = 1; j <= 5; j++)
    {
   for (i = 1; i <= 5; i++)
    printf ("%d ", beenthere[i][j]);

    printf ("\n");
     }


     for (i = 1; i <= 12; i++)
       printf ("%d  ", probs[i]); 

     getch();
	dscreen();
  }

if (choice == ']' && dotalive == 1)
 {
  usepower(havepowerup);
  havepowerup = 0;
  putsprite (0, 580, 40);
  }

 bools2 = 1;
if (choice == 13 && numlives > 0 && dotalive == 0)
{
 for (i = 1; i <= numenemy; i++)
   {
     if (movex == emetype[i].enemyx && movey == emetype[i].enemyy)
      bools2 = 0;
    }
if (bools2 == 1)
{
 numlives--;
 dotalive = 1;
}

 }

if (choice == 'q')
  {
	closegraph();
	quitgame();
  }

if (choice == 27)
  {
    menu();
    }


if (choice == 32 && dotalive == 1 && pstunned == 0)
{
fliptile(movex, movey, 0);
} //end of if for 32





handleenemy();



if (beatlevel != 1)
{
bools = 1;
for (j = 1; j <= maxy; j++)
 for (i = 1; i <= maxx; i++)
   {
     if (map[i][j] == 1)
       bools = 0;
    }
 }

if (bools == 1)
  {

    int tmpit = layoutmap[mapposx][mapposy];

if (tmpit == 4 || tmpit == 32 || tmpit == 33 || tmpit == 34 || tmpit == 22 || tmpit == 23 || tmpit == 24 || tmpit == 11)
   {
   map[8][0] = 8;
   putimg(8, 8*tilex+offsetx, 0+offsety,0);
   }
if (tmpit == 4 || tmpit == 31 || tmpit == 32 || tmpit == 34 || tmpit == 22 || tmpit == 25 || tmpit == 26 || tmpit == 12)
  {
   map[8][12] = 9;
   putimg(9, 8*tilex+offsetx, 12*tiley+offsety,0);
   }

if (tmpit == 4 || tmpit == 31 || tmpit == 32 || tmpit == 33 || tmpit == 21 || tmpit == 23 || tmpit == 25 || tmpit == 14)
   {
   map[0][6] = 10;
   putimg(10, 0*tilex+offsetx, 6*tiley+offsety,0);
   }

if (tmpit == 4 || tmpit == 31 || tmpit == 33 || tmpit == 34 || tmpit == 21 || tmpit == 24 || tmpit == 26 || tmpit == 13)
   {
   map[16][6] = 11;
   putimg(11, 16*tilex+offsetx, 6*tiley+offsety,0);
   }

   beatlevel = 1;

   if (goodtime - timetoplay > 0)
     score = score + 5 * (goodtime - timetoplay);

   for (i = 1; i <= numenemy; i++)
     {
      if (emetype[i].alive == 1) emetype[i].bonus = 48;
      putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
     }
     for (i = 1; i <= numenemy; i++)
      emetype[i].alive = 0;

    bools = 0;

    putsprite(7, movex*tilex+offsetx, movey*tiley+offsety);

      initmusic("O2L8EEEFFFL2GP3L8DDDDD");
      musicplay();

      delay(1000);

   }


}


}


void getimg(int n, char name[30])
{
cleardevice();
int i, j,xx,yy, hold;
char fname[30];
FILE *putter;
strcpy(fname, name);
putter = fopen(fname, "r");
fscanf(putter, "%d %d", &xx, &yy);
for (j = 1; j <= 32; j++)
  for (i = 1; i <= 32; i++)
	 {
	 fscanf (putter, "%d", &hold);
	 putpixel(i-1, j-1, hold);
	 }
 fclose(putter);
unsigned size;
size = imagesize(0,0,31,31);
if ((ptr[n] = farmalloc(size)) == NULL)
{
	closegraph();
	quitgame();
}
 getimage(0,0,31,31,ptr[n]);
}


void putimg(int n, int x, int y, int type)
{
putimage(x, y, ptr[n], type);
}


void getputquick(int x, int y, char name[30])
{
int i, j,xx, yy, hold;
char fname[30];
FILE *putter;
strcpy(fname, name);
putter = fopen(fname, "r");
fscanf(putter, "%d %d", &xx, &yy);
for (j = 1; j <= 32; j++)
  for (i = 1; i <= 32; i++)
	 {
	 fscanf (putter, "%d", &hold);
	 putpixel(i-1+x, j-1+y, hold);
	 }
 fclose(putter);


 }













void dscreen()
{
int i, j;
for (j = 0; j <= maxy; j++)
 for (i = 0; i <= maxx; i++)
 putimg(map[i][j], i*tilex+offsetx, j*tiley+offsety, 0);


 getputquick (580, 80, "lifesign.tsv");
getputquick (580, 115, "pow12.tsv");

 }



void loadmap(char fname[15])
{
void getsprite(int n, char name[15]);
cleardevice();

beenthere[mapposx][mapposy] = 1;



FILE *fp;
fp = fopen(fname, "r");

if (fp == NULL)
 {
	printf ("FILE SCREW UP!!! at map load %s", fname);
	getch();
	cleardevice();
   closegraph();
   fclose(fp);
   quitgame();

 }
else
{


int color;

 for (color=1; color<=15; color++)
    setpalette(color, BLACK);



fscanf (fp, "%d %d\n", &maxx, &maxy);
int i,j;
for (j = 0; j <= maxy; j++)
{
 for (i = 0; i <= maxx; i++)
  {
  fscanf (fp, "%d ", &map[i][j]);
  }
 }

fscanf (fp, "%d", &numtiles);
char tilenames[25][15];

for (i = 0; i <= numtiles-1; i++)
  fscanf (fp, "%s", &tilenames[i]);


for (i = 0; i <= numtiles - 1; i++)
 {
	getimg(i,tilenames[i]);
 }

getimg(8,"brickd1.tsv");
getimg(9,"brickd2.tsv");
getimg(10,"brickd3.tsv");
getimg(11,"brickd4.tsv");



int tempx, tempy, tempn;

 fscanf(fp, "%d", &numenemy);

for (i = 1; i <= numenemy; i++)
 {
   fscanf (fp, "%d %d %d", &tempx, &tempy, &tempn);
   if (tempn == 1)
     emetype[i].enemy = 33;
   if (tempn == 2)
     emetype[i].enemy = 27;
   if (tempn == 3)
     emetype[i].enemy = 21;
   if (tempn == 4)
     emetype[i].enemy = 15;
   if (tempn == 5)
     emetype[i].enemy = 9;



   emetype[i].enemyx = tempx;
   emetype[i].enemyy = tempy;
   emetype[i].alive = 1;
   emetype[i].stun = 1;
   emetype[i].ifstunned = 0;
  emetype[i].waittime = 2;
   if (tempn == 5)
     emetype[i].waittime = 1;
   if (tempn == 1)
     emetype[i].waittime = 3;

   emetype[i].stuntime = 6;

  if (tempn == 5)
   emetype[i].stuntime = 8;
  if (tempn == 4)
   emetype[i].stuntime = 15;
   if (tempn == 2)
   emetype[i].stuntime = 7;



   emetype[i].enemydir = emetype[i].enemy;
  }


 beatlevel = 0;
 powerup = 0;
 havepowerup = 0;
int trys[15];
int holditall;

holditall = 0;

for (i = 0; i <= 12; i++)
   trys[i] = 0;





 for (i = 1; i <= 12; i++)
   {
   fscanf (fp, "%d", &trys[i]);
   holditall = holditall + trys[i];
   }

   fscanf (fp, "%d", &goodtime);

if (holditall != 100)
  {
    graphdefaults();
    printf ("ERROR WITH PROBABLILITY!!");
    fcloseall();
    quitgame();
   }

 for (j = 0; j <= 12; j++)
   probs[j] = 0;


for (j = 0; j <= 12; j++)
  {
  for (i = 0; i <= j; i++)
   {
   probs[j] = probs[j] + trys[i];
   }




   }


 graphdefaults();
 }

 fclose(fp);
   numtilew = 0;
int i, j;
  for (j = 0; j <= maxy; j++)
  for (i = 0; i <= maxx; i++)
   {
    if (map[i][j] == 1)
       numtilew++;
    }

}







void getsprite(int n, char name[15])
{
int i, j,xx,yy;
char fname[30];
FILE *putter;
strcpy(fname, name);
putter = fopen(fname, "r");
fscanf(putter, "%d %d", &xx, &yy);

if ((holdtiles[n] = farmalloc(1100)) == NULL)
  {
    graphdefaults();
    printf ("Ran out at %d!!!", n);
    getch();
    quitgame();
   } 
for (j = 0; j <= 31; j++)
  for (i = 0; i <= 31; i++)
	 fscanf (putter, "%d", &temptile[i][j]);

movedata (FP_SEG(&temptile[0][0]), FP_OFF(&temptile[0][0]), FP_SEG(holdtiles[n]), FP_OFF(holdtiles[n]), 1024);


 fclose(putter);
}

void putsprite(int n, int x, int y)
{
int i, j;

movedata (FP_SEG(holdtiles[n]), FP_OFF(holdtiles[n]), FP_SEG(&temptile[0][0]), FP_OFF(&temptile[0][0]), 1024);

for (j = 0; j <= 31; j++)
  for (i = 0; i <= 31; i++)
	 {
	  if (temptile[i][j] != 16)
		 putpixel(i+x, j+y, temptile[i][j]);
	 }
}



void fliptile(int thex, int they, int thet)
{
int i, j;
if (map[thex][they] == 1)
 {
  map[thex][they] = 2;
  putimg (2, thex*tilex+offsetx, they*tiley+offsety, 0);

  for (i = thex+1; i <= maxx; i++)
    {
      for (j = 1; j <= numenemy; j++)
        {
          if (i == emetype[j].enemyx && they == emetype[j].enemyy)
            {
            emetype[j].stun = emetype[j].stuntime;
            emetype[j].ifstunned = 1;
           putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);

           if (thet == 2) emetype[j].alive = 2;
            }
        }
      if (map[i][they] == 1)
        {
          map[i][they] = 2;
          putimg (2, i*tilex+offsetx, they*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;

     }




   for (i = thex-1; i >= 1; i--)
    {
       for (j = 1; j <= numenemy; j++)
        {
          if (i == emetype[j].enemyx && they == emetype[j].enemyy)
              {
            emetype[j].stun = emetype[j].stuntime;
            emetype[j].ifstunned = 1;
            putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
            if (thet == 2) emetype[j].alive = 2;
            }
        }
      if (map[i][they] == 1)
        {
          map[i][they] = 2;
          putimg (2, i*tilex+offsetx, they*tiley+offsety, 0);
        }
     else
       if (thet == 0 || thet == 2) break;
     }

  for (i = they+1; i <= maxy; i++)
    {
        for (j = 1; j <= numenemy; j++)
        {
          if (thex == emetype[j].enemyx && i == emetype[j].enemyy)
              {
            emetype[j].stun = emetype[j].stuntime;
            emetype[j].ifstunned = 1;
            putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
           if (thet == 2) emetype[j].alive = 2;
            }
        }

      if (map[thex][i] == 1)
        {
          map[thex][i] = 2;
          putimg (2, thex*tilex+offsetx, i*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }
   for (i = they-1; i >= 1; i--)
    {
        for (j = 1; j <= numenemy; j++)
        {
          if (thex == emetype[j].enemyx && i == emetype[j].enemyy)
              {
            emetype[j].stun = emetype[j].stuntime;
            emetype[j].ifstunned = 1;
            putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
           if (thet == 2) emetype[j].alive = 2;
            }
        }

      if (map[thex][i] == 1)
        {
          map[thex][i] = 2;
          putimg (2, thex*tilex+offsetx, i*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }
 }
else
 {
 {
  map[thex][they] = 1;
  putimg (1, thex*tilex+offsetx, they*tiley+offsety, 0);

  for (i = thex+1; i <= maxx; i++)
    {
      if (map[i][they] == 2)
        {
          map[i][they] = 1;
          putimg (1, i*tilex+offsetx, they*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }
   for (i = thex-1; i >= 1; i--)
    {
      if (map[i][they] == 2)
        {
          map[i][they] = 1;
          putimg (1, i*tilex+offsetx, they*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }

  for (i = they+1; i <= maxy; i++)
    {
      if (map[thex][i] == 2)
        {
          map[thex][i] = 1;
          putimg (1, thex*tilex+offsetx, i*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }
   for (i = they-1; i >= 1; i--)
    {
      if (map[thex][i] == 2)
        {
          map[thex][i] = 1;
          putimg (1, thex*tilex+offsetx, i*tiley+offsety, 0);
        }
      else
       if (thet == 0 || thet == 2) break;
     }
 }
}




}





void handleenemy()
{
int i, j;

if (beatlevel == 1)
 {
 for (i = 1; i <= numenemy; i++)
  {
   if (emetype[i].bonus == 48)
     putsprite (emetype[i].bonus, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety);

   if ((emetype[i].enemyx == movex && emetype[i].enemyy == movey) && emetype[i].bonus == 48)
    {
    putimg (map[movex][movey], movex*tilex+offsetx, movey*tiley+offsety, 0);
    emetype[i].bonus = 0;
    score = score + random(100) + 50;
    }

  }

 }

    for (i = 1; i <= numenemy; i++)
{
 if (emetype[i].ifstunned == 1)
    emetype[i].enemydir = emetype[i].enemy + 4; //stunned

if (timetoplay - emetype[i].waitforit >= emetype[i].stun)
{
 emetype[i].waitforit = timetoplay;
 emetype[i].stun = emetype[i].waittime;
 emetype[i].ifstunned = 0;



if (emetype[i].alive == 1)
  {


   if (emetype[i].enemy == 15)
    {

     if ((map[emetype[i].enemyx+1][emetype[i].enemyy] == 2) || (map[emetype[i].enemyx-1][emetype[i].enemyy] == 2) || (map[emetype[i].enemyx][emetype[i].enemyy+1] == 2) || (map[emetype[i].enemyx][emetype[i].enemyy-1] == 2))
      {
       putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
       if(map[emetype[i].enemyx+1][emetype[i].enemyy] == 2)
         {
          emetype[i].enemyx++;
          emetype[i].enemydir = emetype[i].enemy + 1;
          goto out;
         }
       if(map[emetype[i].enemyx-1][emetype[i].enemyy] == 2)
         {
          emetype[i].enemyx--;
          emetype[i].enemydir = emetype[i].enemy + 2;
          goto out;
         }
       if(map[emetype[i].enemyx][emetype[i].enemyy+1] == 2)
         {
          emetype[i].enemyy++;
          emetype[i].enemydir = emetype[i].enemy + 0;
          goto out;
         }
       if(map[emetype[i].enemyx][emetype[i].enemyy-1] == 2)
         {
          emetype[i].enemyy--;
          emetype[i].enemydir = emetype[i].enemy + 3;
          goto out;
         }
       out:
       map[emetype[i].enemyx][emetype[i].enemyy] = 1;
       putimg (1, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
      }

    else
      {
	     emetype[i].rand = random(5);
        if (emetype[i].rand < 4)
          putimg (1, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
        map[emetype[i].enemyx][emetype[i].enemyy] = 1;
        if (emetype[i].rand == 0)
          {
         	 if (map[emetype[i].enemyx][emetype[i].enemyy-1] < 3)
           		emetype[i].enemyy--;
          emetype[i].enemydir = emetype[i].enemy + 3;
          }
        if (emetype[i].rand == 1)
          {
         	 if (map[emetype[i].enemyx][emetype[i].enemyy+1] < 3)
           		emetype[i].enemyy++;
          emetype[i].enemydir = emetype[i].enemy + 0;
          }
       if (emetype[i].rand == 2)
          {
         	 if (map[emetype[i].enemyx-1][emetype[i].enemyy] < 3)
           		emetype[i].enemyx--;
          emetype[i].enemydir = emetype[i].enemy + 2;
          }
       if (emetype[i].rand == 3)
          {
         	 if (map[emetype[i].enemyx+1][emetype[i].enemyy] < 3)
         		emetype[i].enemyx++;
          emetype[i].enemydir = emetype[i].enemy + 1;
          }

        }

       }


if (emetype[i].enemy == 9)
  {

  if (map[emetype[i].enemyx][emetype[i].enemyy] == 2)
    {
      map[emetype[i].enemyx][emetype[i].enemyy] = 1;
      putimg (1, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);

      for (j = emetype[i].enemyy - 1; j >= 1; j--)
        {
          if (map[emetype[i].enemyx][j] == 2)
            {
              map[emetype[i].enemyx][j] = 1;
              putimg (1, emetype[i].enemyx*tilex+offsetx, j*tiley+offsety, 0);
             }
           else
             break;

           if (emetype[i].enemyx == movex && j == movey)
             {
               pstunned = 5;
               playerpic = 6;
             pstun = timetoplay;
             }

         }

      for (j = emetype[i].enemyy + 1; j <= maxy; j++)
        {
          if (map[emetype[i].enemyx][j] == 2)
            {
              map[emetype[i].enemyx][j] = 1;
              putimg (1, emetype[i].enemyx*tilex+offsetx, j*tiley+offsety, 0);
             }
           else
            break;
         if (emetype[i].enemyx == movex && j == movey)
           {
               pstunned = 5;
               playerpic = 6;
             pstun = timetoplay;
             }

         }



      for (j = emetype[i].enemyx - 1; j >= 1; j--)
        {
          if (map[j][emetype[i].enemyy] == 2)
            {
              map[j][emetype[i].enemyy] = 1;
              putimg (1, j*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
             }
            else
              break;


      if (j == movex && emetype[i].enemyy == movey)
            {
               pstunned = 5;
               playerpic = 6;
             pstun = timetoplay;
             }


         }
      for (j = emetype[i].enemyx + 1; j <= maxx; j++)
        {
          if (map[j][emetype[i].enemyy] == 2)
            {
              map[j][emetype[i].enemyy] = 1;
              putimg (1, j*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
             }
            else
              break;
       if (j == movex && emetype[i].enemyy == movey)
            {
               pstunned = 5;
               playerpic = 6;
               pstun = timetoplay;
             }


         }

 }
emetype[i].rand = random(4);
  if (emetype[i].rand < 4)
     putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);

	if (emetype[i].rand == 0)
    {
	 if (map[emetype[i].enemyx][emetype[i].enemyy-1] < 3)
	 emetype[i].enemyy--;
    emetype[i].enemydir = emetype[i].enemy + 3;
    }
if (emetype[i].rand == 1)
  {
	 if (map[emetype[i].enemyx][emetype[i].enemyy+1] < 3)
		emetype[i].enemyy++;
    emetype[i].enemydir = emetype[i].enemy + 0;
  }
if (emetype[i].rand == 2)
  {
	 if (map[emetype[i].enemyx-1][emetype[i].enemyy] < 3)
		emetype[i].enemyx--;
    emetype[i].enemydir = emetype[i].enemy + 2;
  }
if (emetype[i].rand == 3)
  {
	 if (map[emetype[i].enemyx+1][emetype[i].enemyy] < 3)
		emetype[i].enemyx++;
    emetype[i].enemydir = emetype[i].enemy + 1;
  }








   }


if (emetype[i].enemy == 21)
  {
  	emetype[i].rand = random(5);
	if (emetype[i].rand < 4)
     putimg (1, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
   map[emetype[i].enemyx][emetype[i].enemyy] = 1;

	if (emetype[i].rand == 0)
    {
	 if (map[emetype[i].enemyx][emetype[i].enemyy-1] < 3)
	 emetype[i].enemyy--;
    emetype[i].enemydir = emetype[i].enemy + 3;
    }
if (emetype[i].rand == 1)
  {
	 if (map[emetype[i].enemyx][emetype[i].enemyy+1] < 3)
		emetype[i].enemyy++;
    emetype[i].enemydir = emetype[i].enemy + 0;
  }
if (emetype[i].rand == 2)
  {
	 if (map[emetype[i].enemyx-1][emetype[i].enemyy] < 3)
		emetype[i].enemyx--;
    emetype[i].enemydir = emetype[i].enemy + 2;
  }
if (emetype[i].rand == 3)
  {
	 if (map[emetype[i].enemyx+1][emetype[i].enemyy] < 3)
		emetype[i].enemyx++;
    emetype[i].enemydir = emetype[i].enemy + 1;
  }

  }


  if (emetype[i].enemy == 27)
   {
     putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);


     if (emetype[i].enemyy < movey)
       {
          if (map[emetype[i].enemyx][emetype[i].enemyy+1] < 3)
          emetype[i].enemyy++;
          emetype[i].enemydir = emetype[i].enemy + 0;
       }
     if (emetype[i].enemyy > movey)
       {
       if (map[emetype[i].enemyx][emetype[i].enemyy-1] < 3)
          emetype[i].enemyy--;
       emetype[i].enemydir = emetype[i].enemy + 3;
       }

     if (emetype[i].enemyx < movex)
       {
         if (map[emetype[i].enemyx+1][emetype[i].enemyy] < 3)
          emetype[i].enemyx++;
         emetype[i].enemydir = emetype[i].enemy + 1;
       }
     if (emetype[i].enemyx > movex)
       {
         if (map[emetype[i].enemyx -1][emetype[i].enemyy] < 3)
          emetype[i].enemyx--;
        emetype[i].enemydir = emetype[i].enemy + 2;
       }


  }

if (emetype[i].enemy == 33)
  {

    temp = random(3);
   if (temp == 0)
     {
       putimg (1, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);
       map[emetype[i].enemyx][emetype[i].enemyy] = 1;
     }



  	emetype[i].rand = random(5);
	if (emetype[i].rand < 4)
     putimg (map[emetype[i].enemyx][emetype[i].enemyy], emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety, 0);

	if (emetype[i].rand == 0)
    {
	 if (map[emetype[i].enemyx][emetype[i].enemyy-1] < 3)
	 emetype[i].enemyy--;
    emetype[i].enemydir = emetype[i].enemy + 3;
    }
if (emetype[i].rand == 1)
  {
	 if (map[emetype[i].enemyx][emetype[i].enemyy+1] < 3)
		emetype[i].enemyy++;
    emetype[i].enemydir = emetype[i].enemy + 0;
  }
if (emetype[i].rand == 2)
  {
	 if (map[emetype[i].enemyx-1][emetype[i].enemyy] < 3)
		emetype[i].enemyx--;
    emetype[i].enemydir = emetype[i].enemy + 2;
  }
if (emetype[i].rand == 3)
  {
	 if (map[emetype[i].enemyx+1][emetype[i].enemyy] < 3)
		emetype[i].enemyx++;
    emetype[i].enemydir = emetype[i].enemy + 1;
  }




  }





  } // end of ifalive

  }




  } // end ofloop


for (i = 1; i <= numenemy; i++)
  {
  if (emetype[i].alive >= 1)
    {
   if (emetype[i].alive == 2)
     emetype[i].enemydir = emetype[i].enemy + 5;
   if (emetype[i].enemyx == px && emetype[i].enemyy == py && powerup != 0)
     {
      powerup = 0;
      putimg (map[px][py], px*tilex+offsetx, py*tiley+offsety, 0);
      powerappear = timetoplay;
      }


    if (!(emetype[i].enemyx == movex && emetype[i].enemyy == movey)) putsprite (emetype[i].enemydir, emetype[i].enemyx*tilex+offsetx, emetype[i].enemyy*tiley+offsety);
    if ((emetype[i].enemyx == movex && emetype[i].enemyy == movey) && emetype[i].ifstunned != 1 && emetype[i].alive == 1)
       {
    if (emetype[i].enemy == 27 && emetype[i].alive == 1)
       {
       emetype[i].alive = 0;
       emetype[i].enemyx = 1;
       emetype[i].enemyy = 1;
       }
       dotalive = 0;
       playerpic = 5;
       }

     }
   }

}





void usepower(int thep)
{
thep = thep - 4;
int i, j, tempx, tempy;
if (thep == 35)
 {
   for (j = 1; j <= maxy; j++)
    for (i = 1; i <= maxx; i++)
      {
        if (map[i][j] == 1)
          map[i][j] = 2;
        else if (map[i][j] == 2)
           map[i][j] = 1;
      putimg (map[i][j], i*tilex+offsetx, j*tiley+offsety, 0);
       }
  }

if (thep == 36)
  {
    for (i = 1; i <= random(30)+10; i ++)
      {
        tempx = 0;
        tempy = 0;
        while (map[tempx][tempy] > 2)
          {
          tempx = random(maxx);
          tempy = random(maxy);
          }

          map[tempx][tempy] = 2;

     putimg (map[tempx][tempy], tempx*tilex+offsetx, tempy*tiley+offsety, 0);
       }
     }
    if (thep == 37)
      {
        for (j = 1; j <= numenemy; j++)
          {
          emetype[j].stun = emetype[j].stuntime;
          emetype[j].ifstunned = 1;
          putimg (map[emetype[j].enemyx][emetype[j].enemyy], emetype[j].enemyx*tilex+offsetx, emetype[j].enemyy*tiley+offsety, 0);
          }

       }

     if (thep == 40)
     {
        for (i = 1; i <= random(7)+3; i++)
         {
         tempx = 0;
         tempy = 0;
          while (map[tempx][tempy] > 2)
           {
           tempx = random(maxx);
           tempy = random(maxy);
           }
           for (j = 1; j <= numenemy; j++)
            {
             if (emetype[j].enemyx == tempx && emetype[j].enemyy == tempy)
               emetype[j].alive = 0;
            }
           if (map[tempx][tempy] == 1)
             fliptile(tempx, tempy, 0);
          }
        }

    if (thep == 43)
      fliptile (movex, movey, 1);

   if (thep == 41)
     fliptile (movex, movey, 2);









 }








void initgame()
{

int color;

 for (color=1; color<=15; color++)
    setpalette(color, BLACK);





FILE *fp;
int i, j;
fp = fopen("map.lyt", "r");
for (j = 1; j <= 5; j++)
 for (i = 1; i <= 5; i++)
  fscanf (fp, "%d", &layoutmap[i][j]);

fscanf (fp, "%d %d %d %d", &exitposx, &exitposy, &exitmapx, &exitmapy);
for (j = 1; j <= 5; j++)
 for (i = 1; i <= 5; i++)
  fscanf (fp, "%d", &levellookup[i][j]);



fclose(fp);
    getsprite(0, "blank.tsv");
    getsprite(1, "dot1.tsv");
	 getsprite(2, "dot2.tsv");
	 getsprite(3, "dot3.tsv");
	 getsprite(4, "dot4.tsv");
    getsprite(5, "dot5.tsv");
    getsprite(6, "dot6.tsv");
    getsprite(7, "dot7.tsv");
    getsprite (9, "phil1.tsv");
    getsprite (10, "phil2.tsv");
    getsprite (11, "phil3.tsv");
    getsprite (12, "phil4.tsv");
    getsprite (13, "phil5.tsv");
    getsprite (14, "phil6.tsv");
	 getsprite (15, "bork1.tsv");
    getsprite (16, "bork2.tsv");
    getsprite (17, "bork3.tsv");
    getsprite (18, "bork4.tsv");
    getsprite (19, "bork5.tsv");
    getsprite (20, "bork6.tsv");
    getsprite (21, "borkii1.tsv");
    getsprite (22, "borkii2.tsv");
    getsprite (23, "borkii3.tsv");
    getsprite (24, "borkii4.tsv");
    getsprite (25, "borkii5.tsv");
    getsprite (26, "borkii6.tsv");
    getsprite (27, "bot1.tsv");
    getsprite (28, "bot2.tsv");
    getsprite (29, "bot3.tsv");
    getsprite (30, "bot4.tsv");
    getsprite (31, "bot5.tsv");
    getsprite (32, "bot6.tsv");
    getsprite (33, "edot1.tsv");
    getsprite (34, "edot2.tsv");
    getsprite (35, "edot3.tsv");
    getsprite (36, "edot4.tsv");
    getsprite (37, "edot5.tsv");
    getsprite (38, "edot6.tsv");
    getsprite (39, "pow1.tsv");
    getsprite (40, "pow2.tsv");
    getsprite (41, "pow3.tsv");
    getsprite (42, "pow4.tsv");
    getsprite (43, "pow5.tsv");
    getsprite (44, "pow6.tsv");
    getsprite (45, "pow7.tsv");
    getsprite (46, "pow8.tsv");
    getsprite (47, "pow9.tsv");
    getsprite (48, "pow10.tsv");
    getsprite (49, "pow11.tsv");
    getsprite (50, "pow12.tsv");
    
  graphdefaults();

}

void menu()
{
int i, j;

for (i = 220; i >= 0; i--)
{
setcolor(8);
rectangle(0+offsetx+i,0+offsety+i,17*tilex+offsetx-i,13*tiley+offsety-i);
setcolor(0);
rectangle(0+offsetx+i+10,0+offsety+i+10,17*tilex+offsetx-i-10,13*tiley+offsety-i-10);

}


char thename[20];
float temptile = 0;
for (j = 1; j <= 5; j++)
  for (i = 1; i <= 5; i++)
    {
       if (beenthere[i][j] == 1)
       {
       itoa (layoutmap[i][j], thename, 10);
       strcat(thename, ".tsv");
       }
      else
        strcpy(thename, "0.tsv");

       getputquick(30+i*32+offsetx,30+j*32+offsety, thename);
    }
    setfillstyle(1,14);
    setcolor(14);
    fillellipse(46+mapposx*32+offsetx,46+mapposy*32+offsety, 5,5);

    setcolor(15);
    outtextxy(320+offsetx,60+offsety, "LAST POWERUP");

    rectangle(348+offsetx, 78+offsety, 383+offsetx, 112+offsety);

    if (oldpowerup != 0)
    {
   //   printf ("%d", oldpowerup);
      putsprite(oldpowerup, 350+offsetx, 80+offsety);
    }

    getputquick(350+offsetx, 130+offsety, "lifesign.tsv");

    itoa(numlives, thename, 10);
    outtextxy (385+offsetx, 155+offsety, thename);

    getputquick(350+offsetx, 170+offsety, "pow12.tsv");
    itoa(numkey, thename, 10);
    outtextxy (385+offsetx, 185+offsety, thename);




    temptile = 0;
 for (j = 0; j <= maxy; j++)
  for (i = 0; i <= maxx; i++)
    if (map[i][j] == 2)
        temptile++;


  temptile = (temptile  / numtilew * 100);

  itoa(temptile, thename, 10);
    strcat(thename, " %");
    outtextxy (315+offsetx, 220+offsety, "PERCENT DONE:");
    outtextxy (355+offsetx, 230+offsety, thename);

    char temptext[30];

  strcpy(temptext, "LOCATION X: ");
  itoa(movex, thename, 10);
  strcat (temptext, thename);
  outtextxy (315+offsetx, 280+offsety, temptext);

  strcpy(temptext, "LOCATION Y: ");
  itoa(movey, thename, 10);
  strcat (temptext, thename);
  outtextxy (315+offsetx, 290+offsety, temptext);

  outtextxy (300+offsetx, 300+offsety, "ELAPSED LEVEL TIME: ");
  itoa(timetoplay, temptext, 10);
  strcat (temptext, " SECONDS");
  outtextxy (320+offsetx, 310+offsety, temptext);

  outtextxy (300+offsetx, 320+offsety, "  BENCHMARK TIME: ");
  itoa(goodtime, temptext, 10);
  strcat (temptext, " SECONDS");
  outtextxy (320+offsetx, 330+offsety, temptext);

  setcolor(14);
  setfillstyle(1,14);
  fillellipse(480,180,30,30);
  setcolor(7);
  setfillstyle(1,7);
  fillellipse(480,180,28,28);
  setcolor(15);
  outtextxy(476,155, "N");

  int temx = 0, temy = 0;
  if (exitposx < mapposx)
     temx = -15;
   if (exitposx > mapposx)
     temx = 15;
   if (exitposy < mapposy)
     temy = -20;
   if (exitposy > mapposy)
     temy = 20;
   setlinestyle(0, 0, 3);
    setcolor(0);
  fillellipse(480,180,2,2);
 if (havecomp == 1)
    line(480,180,480+temx, 180+temy);
   havecomp = 0;
   setlinestyle(0, 0, 1);

  strcpy(temptext, "SCORE:  ");
  itoa(score, thename, 10);
  strcat (temptext, thename);
  outtextxy (100+offsetx, 40+offsety, temptext);



    int option = 1;
    char opchoice;

  if (option == 1)
     setcolor(15);
  else
    setcolor(8);
   outtextxy (109+offsetx, 300+offsety, "CONTINUE");

   if (option == 2)
     setcolor(15);
  else
    setcolor(8);
   outtextxy (109+offsetx, 320+offsety, "  QUIT  ");



 opchoice = 0;
 while (opchoice != 13 && opchoice != 27)
 {
  opchoice = getch();

 if (opchoice == 72)
   {
     option--;
     if (option < 1) option++;
   }

  if (opchoice == 80)
   {
     option++;
     if (option > 2) option--;
   }


  if (option == 1)
     setcolor(15);
  else
    setcolor(8);
   outtextxy (109+offsetx, 300+offsety, "CONTINUE");

   if (option == 2)
     setcolor(15);
  else
    setcolor(8);
   outtextxy (109+offsetx, 320+offsety, "  QUIT  ");
  }

   if (option == 2)
     {
       cleardevice();
       closegraph();
       quitgame();
     }
    dscreen();
    gametime = time(NULL);
    }



void win()
{
void box(int x, int y, int x2, int y2, int c, int f);

cleardevice();
setfillstyle(1,4);
floodfill (1, 1, 15);
int i, j, a;
setcolor(8);
for (i = 10; i <= 480; i=i+50)
  line(0, i, 640, i);


a = 0;
for (j = 10; j <= 480; j=j+50)
{

if (a % 2 == 0)
  {
   for (i = -60; i <= 640; i = i + 120)
     line (i, j, i, j + 50);
   }
else
{
 for (i = 0; i <= 640; i = i + 120)
   line (i, j, i, j + 50);
}
a++;

}


setcolor(8);
setfillstyle(1,8);
fillellipse(320,140,120,120);
box (200, 140, 440, 460, 8, 1);

setcolor(6);
setfillstyle(1,6);
fillellipse(320,140,110,110);
box (210, 150, 430, 450, 6, 1);

box (210, 170, 430, 180, 0, 1);
box (210, 370, 430, 380, 0, 1);

box(370,250,410,310,7,1);
setcolor(0);
setfillstyle(1,0);
fillellipse(390, 267,8,8);
box(391,262,393,264,15,1);
setcolor(0);
fillellipse(390, 290,5,5);
box(388,294,392,302,0,1);


 gotoxy(1,1);
 setcolor(15);


box (0, 0, 160, 50, 4, 1);
setcolor(8);
line (0, 10, 200, 10);
line (60, 10, 60, 50);


delay(2000);

setcolor(0);
setfillstyle(1,0);
fillellipse(320,140,110,110);
box (210, 150, 430, 450, 0, 1);

setcolor(15);
circle(360,200,30);
circle(360,290,60);
circle(326,200,5);
box(325,197,336,203,0,1);
setcolor(15);
line(300,290,300,380);
box(301,290,420,380,0,1);
box(360,160,420,380,0,1);

setcolor(15);
 // settextjustify(1, 1);
  outtextxy(250,90,"CONGRATULATIONS!");
  outtextxy(250,110,"YOU HAVE ESCAPED");
  outtextxy(250,120,"FROM MY MAZE...");
  delay(1000);
  outtextxy(248,400,"LET'S SEE HOW YOU");
  outtextxy(239,410,"FAIR IN MY DUNGEON.");
  outtextxy(245,420, "     HA HA HA");


delay(2000);

outtextxy(230,210, "   THE END   ");
getch();
           closegraph();

     

 quitgame();

 }


int getprob()
{
int hold, i;

hold = random(100) + 1;
for (i = 1; i <= 12; i++)
  if (hold >= probs[i - 1] && hold < probs[i] + 1)
    {
      hold = i - 1;
      break;
    }

return hold;
}


void start()
{
void box(int x, int y, int x2, int y2, int c, int f);
void instruct();


int color;

tops:

 for (color=1; color<=15; color++)
    setpalette(color, BLACK);

cleardevice();
setfillstyle(1,4);
floodfill (1, 1, 15);
int i, j, a;
setcolor(8);
for (i = 10; i <= 480; i=i+50)
  line(0, i, 640, i);


a = 0;
for (j = 10; j <= 480; j=j+50)
{

if (a % 2 == 0)
  {
   for (i = -60; i <= 640; i = i + 120)
     line (i, j, i, j + 50);
   }
else
{
 for (i = 0; i <= 640; i = i + 120)
   line (i, j, i, j + 50);
}
a++;

}


setcolor(8);
setfillstyle(1,8);
fillellipse(320,140,120,120);
box (200, 140, 440, 460, 8, 1);

setcolor(6);
setfillstyle(1,6);
fillellipse(320,140,110,110);
box (210, 150, 430, 450, 6, 1);

box (210, 170, 430, 180, 0, 1);
box (210, 370, 430, 380, 0, 1);

box(370,250,410,310,7,1);
setcolor(0);
setfillstyle(1,0);
fillellipse(390, 267,8,8);
box(391,262,393,264,15,1);
setcolor(0);
fillellipse(390, 290,5,5);
box(388,294,392,302,0,1);


 gotoxy(1,1);
 setcolor(15);
   settextjustify(CENTER_TEXT, CENTER_TEXT);
    settextstyle(4, HORIZ_DIR, 5);
    outtextxy(320,40,"MAZE GAME 1/oo");
    outtextxy(320,80,"BASIC TRAINING");
    settextstyle(0, HORIZ_DIR, 1);
    settextjustify(0, 0);
    int center;
graphdefaults();
char opit;
opit = 0;
int choos = 1;
choos = 1;
fflush(stdin);

while (opit != 13)
{


center = 320 - (10*8);
if (choos == 1)
{
setcolor(15);
outtextxy(center+40,210,"ENTER MAZE");
}
else
{
setcolor(8);
outtextxy(center+41,211,"ENTER MAZE");
setcolor(7);
outtextxy(center+40,210,"ENTER MAZE");
}

if (choos == 2)
{
center = 320 - (10*8);
setcolor(15);
outtextxy(center+35,230,"INSTRUCTION");
}
else
{
center = 320 - (10*8);
setcolor(8);
outtextxy(center+35+1,230+1,"INSTRUCTION");
setcolor(7);
outtextxy(center+35,230,"INSTRUCTION");
}

if (choos == 3)
{
center = 320 - (10*4);
setcolor(15);
outtextxy(center+25,250,"FLEE");

}
else
{

center = 320 - (10*4);
setcolor(8);
outtextxy(center+25+1,250+1,"FLEE");
setcolor(7);
outtextxy(center+25,250,"FLEE");
}

opit = getch();

if (opit == 72)
 {
   choos--;
   if (choos < 1) choos = 1;
  }
if (opit == 80)
 {
   choos++;
   if (choos > 3) choos = 3;
  }


}



   if (choos == 0)
       goto tops;



   if (choos == 2)
     {
       instruct();
       cleardevice();
       goto tops;
     }

      if (choos == 3)
     {
       cleardevice();
       closegraph();
       quitgame();
     }


      }


void box(int x, int y, int x2, int y2, int c, int f)
{
setcolor(c);

  if (f == 0)
    {
      line(x,y,x2,y);
      line(x2,y,x2,y2);
      line(x2,y2,x,y2);
      line(x,y2,x,y);
    }
  else
    {
     int okay;
       for (okay = x; okay <= x2; okay++)
         line(okay,y,okay,y2);
    }







}







void instruct()
{

struct palettetype pal;
 getpalette(&pal);
int i;

   for (i=0; i<16; i++)
      setrgbpalette(pal.colors[i], i*4, i*4, i*4);



  
   int j;
   int k;

   cleardevice();

   char message[100];
   char holdit[100];

 for (k = 1; k <= 43; k++)
 {
   if (k == 1) strcpy(message, "MISSION BRIEFING:");
   if (k == 2) strcpy(message, " ");
   if (k == 3) strcpy(message, "YOU MUST ESCAPE THE MAZE THAT I, HITCH, HAVE CREATED TO TEST AND ANNOY YOU.");
   if (k == 4) strcpy(message, "EACH ROOM IS PART OF THE LARGER MAZE THAT YOU MUST SOLVE.  WHEN YOU GAIN EXIT");
   if (k == 5) strcpy(message, "FROM EACH ROOM, THERE MAY BE 4 WAYS TO GO, PERHAPS 3, 2 OR 1.  TO LEAVE EACH ");
   if (k == 6) strcpy(message, "ROOM YOU MUST FLIP ALL THE FLOOR TILES FROM THIER NATIVE STATE TO THIER OTHER");
   if (k == 7) strcpy(message, "SIDE, ONCE THE ENTIRE FLOOR IS SWITCHED, YOU MAY MOVE ON.");
   if (k == 8) strcpy(message, " ");
   if (k == 9) strcpy(message, "HA HA HA, YOU THINK IT'S EASY, DON'T YOU.  WELL, THERE ARE AN ASSORTMENT OF MY");
   if (k == 10) strcpy(message, "MINIONS IN EACH ROOM, EITHER TRYING TO FLIP THE FLOOR TILES BACK TO THIER ");
   if (k == 11) strcpy(message, "ORIGINAL STATE, OR TRYING TO, WELL, KILL YOU.");
   if (k == 12) strcpy(message, " ");
   if (k == 13) strcpy(message, "TO FLIP TILES, PRESS THE SPACEBAR, AND THE TILE YOU ARE ON, AND ALL THE TILES");
   if (k == 14) strcpy(message, "EXTENDING UP, DOWN, LEFT AND RIGHT OF YOU WILL CHANGE AS WELL UNTIL THEY HIT");
   if (k == 15) strcpy(message, "A CHANGED TILE OR A WALL.  IF YOU SET OFF A LINE OF TILES WHERE AN ENEMY IS ");
   if (k == 16) strcpy(message, "PRESENT, IT WILL BE STUNED FOR A FEW SECONDS, YOU MAY MAKE USE OF THIS TIME TO ");
   if (k == 17) strcpy(message, "FLIP AS MANY TILES AS YOU CAN.  TOUCHING AN ENEMY CAN BE FATAL SO WATCH OUT! ");
   if (k == 18) strcpy(message, " ");
   if (k == 19) strcpy(message, "THERE IS AN ASSORTMENT OF POWERUPS YOU CAN OBTAIN, THEY APPEAR RANDOMLY AT ");
   if (k == 20) strcpy(message, "RANDOM LOCATIONS.  THEY CAN HELP YOU FLIP THE TILES, OR HURT YOU, KILL THE ");
   if (k == 21) strcpy(message, "ENEMIES, OR HELP THEM, MAKE SURE YOU DON'T PICK UP THE BAD ONES, BUT  ");
   if (k == 22) strcpy(message, "ENEMIES CAN DESTROY IT, SO MAKE SURE YOU ARE THE FIRST TO GET TO ITS LOCATION.");
   if (k == 23) strcpy(message, "EACH POWERUP IS DENOTED BY ITS PICTURE, SO MEMORIZE THEM WHILE PLAYING. ");
   if (k == 24) strcpy(message, "WHEN YOU PICK ONE UP, THE TIME YOU HAVE TO USE IT IS LIMITED.");
   if (k == 25) strcpy(message, "IF YOU RUN OUT OF TIME, IT WILL BE LOST.  THE TIME YOU HAVE IS");
   if (k == 26) strcpy(message, "BASED ON YOUR CURRENT SCORE.  THERE IS A POINT WHERE YOU HAVE UNLIMITED TIME");
   if (k == 27) strcpy(message, "TO FIND A GOOD PLACE FOR THE POWERUP'S USE.  THERE ARE A FEW POWERUPS THAT");
   if (k == 28) strcpy(message, "DON'T DELAY, THEY ARE USED AUTOMATICALLY ON PICKUP, SUCH AS 1UP'S.");
   if (k == 29) strcpy(message, "TO USE A POWERUP, PRESS ] AND SEE WHAT HAPPENS. ");
   if (k == 30) strcpy(message, " ");
   if (k == 31) strcpy(message, "YOU MAY PRESS ESC TO BRING UP THE STAT SCREEN, TO SEE WHERE YOU ARE IN MY ");
   if (k == 32) strcpy(message, "DUNGEON AND HOW MUCH OF THE LEVEL YOU HAVE COMPLETED.   ");
   if (k == 33) strcpy(message, " ");
   if (k == 34) strcpy(message, "THE SCORE EFFECTS THE LENGTH OF TIME YOU CAN HOLD A POWERUP, BUT IF YOU CAN");
   if (k == 35) strcpy(message, "GET YOUR SCORE HIGH ENOUGH, YOU MAY ALSO GET A SPECIAL PRIZE.  YOU CAN RAISE ");
   if (k == 36) strcpy(message, "YOUR SCORE BY PICKING UP GEMS, DIAMONDS AND BEATING THE BENCHMARK TIME IN A  ");
   if (k == 37) strcpy(message, "LEVEL, EACH SECOND ADDS 5 TO YOUR SCORE. ");
   if (k == 38) strcpy(message, " ");
   if (k == 39) strcpy(message, "FARE WELL, MY YOUNG DOT, IF MANAGE TO FIND THE EXIT OF THE CATACOMBS, YOU MAY");
   if (k == 40) strcpy(message, "BE WORTHY OF FIGHTING ON MY MAZE WORLD... ");
   if (k == 41) strcpy(message, " ");
   if (k == 42) strcpy(message, " ");
   if (k == 43) strcpy(message, "LET THE TRAINING BEGIN!!!.........         ");
   for (i = 0; i <= strlen(message)-1; i++)
     {
      holdit[0] = message[i];
      holdit[1] = '\0';
     for (j = 0; j <= 15; j++)
      {
      setcolor(j);
      outtextxy (i*8+6,k*9+1,holdit);
      delay(2);


      }

       if (kbhit())
        goto outs;

   }
  }


outs:

   getch();
  for (i=63; i>=0; i--)
    {
    setrgbpalette(pal.colors[15], 63-i, i, i);
      delay(2);
    }

int rr, xx, yy;
 randomize();
 void far *ptr;
unsigned size;

size = imagesize(0, 0, 24, 24);
ptr = farmalloc(size);

for (rr = 1; rr <= 20000; rr++)
{

    xx = random(616);
    yy = random(480);
    getimage(xx, yy, xx+24, yy+24, ptr);
    putimage(xx, yy + 2, ptr, COPY_PUT);

     if (kbhit())
        {
         xx = getch();
        break;
         }
}
fflush(stdin);
cleardevice();


closegraph();
int gdriver = DETECT, gmode;
initgraph(&gdriver, &gmode, "");
farfree(ptr);
cleardevice();

putsprite (33, 1*32, 0*32);
putsprite (34, 2*32, 0*32);
putsprite (35, 1*32, 1*32);
putsprite (36, 2*32, 1*32);

putsprite (21, 1*32, 3*32);
putsprite (22, 2*32, 3*32);
putsprite (23, 1*32, 4*32);
putsprite (24, 2*32, 4*32);

putsprite (15, 1*32, 6*32);
putsprite (16, 2*32, 6*32);
putsprite (17, 1*32, 7*32);
putsprite (18, 2*32, 7*32);

putsprite (27, 1*32, 9*32);
putsprite (28, 2*32, 9*32);
putsprite (29, 1*32, 10*32);
putsprite (30, 2*32, 10*32);

putsprite (9, 1*32, 12*32);
putsprite (10, 2*32, 12*32);
putsprite (11, 1*32, 13*32);
putsprite (12, 2*32, 13*32);


setcolor(15);

outtextxy(4*32, 20+0*32, "PURPLE DOT: a subversion of one of your own kind.  Not");
outtextxy(4*32, 30+0*32, "particularly skilled in the hunting and killing buisness");
outtextxy(4*32, 40+0*32, "thus, he is very slow, and doesn't always flip tiles back.");

outtextxy(4*32, 20+3*32, "GREEN BORK: Once of Hitches Minions, he is a difference species");
outtextxy(4*32, 30+3*32, "of the same BORK race.  He is slower and less intelligent that");
outtextxy(4*32, 40+3*32, "his counterpart, he flips as he walks, but he walks randomly.");

outtextxy(4*32, 20+6*32, "RED BORK: The 'smarter' bork species.  He is intelligent and");
outtextxy(4*32, 30+6*32, "deadly, he will seek out flipped tiles and systematically flip");
outtextxy(4*32, 40+6*32, "them back, he will also attack the others if they are close.");

outtextxy(4*32, 20+9*32, "BOMBOT: A perverted creation of Hitch, a mindless drone that");
outtextxy(4*32, 30+9*32, "hovers around the maze blindly chasing you.  He will follow");
outtextxy(4*32, 40+9*32, "untill he hits his target, at which point it deactivates.");

outtextxy(4*32, 20+12*32, "PHIL:  Another creation, but this a lot more intelligent and ");
outtextxy(4*32, 30+12*32, "evil.  This wanders around randomly, but near it's position, ");
outtextxy(4*32, 40+12*32, "and flips tiles like you, in 4 directions, and stuns.");

getch();

cleardevice();

outtextxy(300, 1*32, "POWERUPS");

putsprite (39, 1*32, 2*32);
putsprite (40, 1*32, 3*32);
putsprite (41, 1*32, 4*32);
putsprite (42, 1*32, 5*32);
putsprite (43, 1*32, 6*32);
putsprite (44, 1*32, 7*32);
putsprite (45, 1*32, 8*32);
putsprite (46, 1*32, 9*32);
putsprite (47, 1*32, 10*32);
putsprite (48, 1*32, 11*32);
putsprite (49, 1*32, 12*32);
putsprite (50, 1*32, 13*32);


outtextxy (3*32, 2*32+12, "INVERT:  Inverts all the tiles on the game grid.");
outtextxy (3*32, 3*32+12, "RAIN: Flips a few random tiles to the flipped position.");
outtextxy (3*32, 4*32+12, "STORM: Freezes all enemies on  the level.");
outtextxy (3*32, 5*32+12, "1 UP: Gives you another chance to win.");
outtextxy (3*32, 6*32+12, "COMPASS: Menu will show vague direction of exit once.");
outtextxy (3*32, 7*32+12, "BOULDER: Like RAIN, but flips the tiles in the 4way spread.");
outtextxy (3*32, 8*32+12, "KILL: 4way spread that kills enemies if it intercepts them.");
outtextxy (3*32, 9*32+12, "RANDOM: A random powerup is given on aquisition.");
outtextxy (3*32, 10*32+12, "PHANTOM SPREAD: 4way spread that goes through obstacles.");
outtextxy (3*32, 11*32+12, "GEM:  On aquisition, you gain some points.");
outtextxy (3*32, 12*32+12, "DIAMOND:  On aquisition, you gain many points.");
outtextxy (3*32, 13*32+12, "KEY: Adds a key to inventory, keys can open locked doors.");
getch();




}


void musicplay()
{
char currlet;

if (musictype.anymusic == 1)
{

while (musictype.notealong < musictype.musiclen)
{
currlet = music[musictype.notealong];

if (currlet >= 'A' && currlet <= 'G')
{



//'C = 131
//'D = 147
//'E = 165
//'F = 175
//'G = 195
//'A = 220
//'B = 247

if (currlet == 'A')
  musictype.freq = 220-65;
if (currlet == 'B')
  musictype.freq = 247-65;
if (currlet == 'C')
  musictype.freq = 131-65;
if (currlet == 'D')
  musictype.freq = 147-65;
if (currlet == 'E')
  musictype.freq = 165-65;
if (currlet == 'F')
  musictype.freq = 175-65;
if (currlet == 'G')
  musictype.freq = 195-65;

sound(musictype.freq + 65 * musictype.noteoct);
delay(musictype.notelen);
nosound();
if (musictype.delays > 0)
  delay(musictype.delays);


}
else
{

if (currlet == '>')
  musictype.noteoct++;

if (currlet == '<')
  musictype.noteoct--;


 if (currlet == 'O')
   {
     musictype.noteoct = music[musictype.notealong + 1] - 48;
     musictype.notealong++;

   }
 if (currlet == 'L')
   {
     musictype.notelen = music[musictype.notealong + 1] - 48;
if (musictype.notelen == 1)
     musictype.notelen = 1648;
if (musictype.notelen == 2)
     musictype.notelen = 824;
if (musictype.notelen == 3)
     musictype.notelen = 549;
if (musictype.notelen == 4)
     musictype.notelen = 439;
if (musictype.notelen == 5)
     musictype.notelen = 329;
if (musictype.notelen == 6)
     musictype.notelen = 274;
if (musictype.notelen == 7)
     musictype.notelen = 219;
if (musictype.notelen == 8)
     musictype.notelen = 164;


     musictype.notealong++;


    }
 if (currlet == 'W')
   {
     musictype.delays = music[musictype.notealong + 1] - 48;
if (musictype.delays == 1)
     musictype.delays = 1648;
if (musictype.delays == 2)
     musictype.delays = 824;
if (musictype.delays == 3)
     musictype.delays = 549;
if (musictype.delays == 4)
     musictype.delays = 439;
if (musictype.delays == 5)
     musictype.delays = 329;
if (musictype.delays == 6)
     musictype.delays = 274;
if (musictype.delays == 7)
     musictype.delays = 219;
if (musictype.delays == 8)
     musictype.delays = 164;



          musictype.notealong++;

    }
  if (currlet == 'P')
    {

     musictype.pause = (music[musictype.notealong + 1] - 48);

//'1 = 30
//'2 = 15
//'3 = 10
//'4 = 8
//'5 = 6
//'6 = 5
//'7 = 4
//'8 = 3

if (musictype.pause == 1)
     musictype.pause = 1648;
if (musictype.pause == 2)
     musictype.pause = 824;
if (musictype.pause == 3)
     musictype.pause = 549;
if (musictype.pause == 4)
     musictype.pause = 439;
if (musictype.pause == 5)
     musictype.pause = 329;
if (musictype.pause == 6)
     musictype.pause = 274;
if (musictype.pause == 7)
     musictype.pause = 219;
if (musictype.pause == 8)
     musictype.pause = 164;



     delay(musictype.pause);


     musictype.notealong++;

   }


  }
  musictype.notealong++;

  }
 }
}





void musicnotestep()
{
char currlet;

if (musictype.anymusic == 1)
{


if (musictype.notealong < musictype.musiclen)
{
currlet = music[musictype.notealong];

if (currlet >= 'A' && currlet <= 'G')
{

if (currlet == 'A')
  musictype.freq = 220-65;
if (currlet == 'B')
  musictype.freq = 247-65;
if (currlet == 'C')
  musictype.freq = 131-65;
if (currlet == 'D')
  musictype.freq = 147-65;
if (currlet == 'E')
  musictype.freq = 165-65;
if (currlet == 'F')
  musictype.freq = 175-65;
if (currlet == 'G')
  musictype.freq = 195-65;

sound(musictype.freq + 65 * musictype.noteoct);
delay(musictype.notelen);
nosound();
if (musictype.delays > 0)
  delay(musictype.delays);


}
else
{
 if (currlet == 'O')
   {
     musictype.noteoct = music[musictype.notealong + 1] - 48;
     musictype.notealong++;

   }
 if (currlet == 'L')
   {
     musictype.notelen = (music[musictype.notealong + 1] - 48) * 100 + (music[musictype.notealong + 2] - 48) * 10;
     musictype.notealong++;
     musictype.notealong++;

    }
 if (currlet == 'W')
   {
     musictype.delays = (music[musictype.notealong + 1] - 48) * 10 + (music[musictype.notealong + 2] - 48);
     musictype.notealong++;
     musictype.notealong++;

    }
  if (currlet == 'P')
    {
     delay( ((music[musictype.notealong + 1] - 48) * 100) + ((music[musictype.notealong + 2] - 48) * 10));

     musictype.notealong++;
     musictype.notealong++;

   }


  }
  musictype.notealong++;

  }
 }
}



void initmusic(char temp[200])
{
strcpy(music, temp);

musictype.notealong = 0;
musictype.musiclen = strlen(music);
musictype.noteoct = 0;
musictype.notelen = 200;
musictype.delays = 0;



}






void quitgame()
{
int i;
for (i = 0; i < 100; i++)
  if (holdtiles[i] != NULL)
    farfree(holdtiles[i]);


for (i = 0; i < 50; i++)
  if (ptr[i] != NULL)
    farfree(ptr[i]);

exit(0);

}
